<?php

declare(strict_types=1);

namespace Drupal\Tests\date_point\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use WebDriver\Exception\NoSuchElement;

/**
 * @group date_point
 *
 * @method \Drupal\FunctionalJavascriptTests\WebDriverWebAssert assertSession()
 */
abstract class TestBase extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'claro';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['date_point', 'node', 'field_ui', 'date_point_test'];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->createContentType(['type' => 'page', 'name' => 'Page']);
    $permissions = [
      'administer content types',
      'administer node fields',
      'create page content',
      'administer node form display',
      'administer node display',
      'delete any page content',
      'edit any page content',
    ];
    $this->drupalLogin(
      $this->drupalCreateUser($permissions),
    );

    $this->config('system.date')->set('timezone.default', 'UTC')->save();
    $this->config('system.date')->set('timezone.user.configurable', FALSE)->save();
  }

  /**
   * {@selfdoc}
   */
  final protected function createField(string $precision = 'seconds'): void {
    $driver = $this->getSession()->getDriver();
    $assert_session = $this->assertSession();
    $this->drupalGet('/admin/structure/types/manage/page/fields/add-field');
    $assert_session->elementExists('xpath', '//h1[text() = "Add field"]');
    $driver->click('//input[@name = "new_storage_type" and @value = "date_time"]');
    $assert_session->assertWaitOnAjaxRequest();
    $driver->setValue('//input[@name = "label"]', 'Example');
    $driver->setValue('//input[@name = "group_field_options_wrapper"]', 'date_point');
    $driver->click('//input[@value = "Continue"]');
    if ($precision !== 'seconds') {
      $driver->selectOption('//fieldset[@id = "edit-field-storage-subform-settings-precision--wrapper"]//input', $precision);
      $assert_session->assertWaitOnAjaxRequest();
    }
    $driver->click('//input[@value = "Save settings"]');
    $assert_session->statusMessageContains('Saved Example configuration.');
  }

  /**
   * {@selfdoc}
   */
  final protected function updateField(string $precision = 'seconds', string $behavior = 'manual'): void {
    $this->deleteAllNodes();
    $driver = $this->getSession()->getDriver();
    $assert_session = $this->assertSession();
    $this->drupalGet('/admin/structure/types/manage/page/fields/node.page.field_example');
    $assert_session->elementExists('xpath', '//h1[. = "Example settings for Page"]');
    if ($precision !== 'seconds') {
      $driver->selectOption('//fieldset[@id = "edit-field-storage-subform-settings-precision--wrapper"]//input', $precision);
      $assert_session->assertWaitOnAjaxRequest();
    }
    $driver->selectOption('//fieldset[@data-drupal-selector = "edit-settings-behavior"]//input', $behavior);
    $driver->click('//input[@value = "Save settings"]');
    $assert_session->statusMessageContains('Saved Example configuration.');
  }

  /**
   * {@selfdoc}
   */
  final protected function configureWidget(string $widget): void {
    $driver = $this->getSession()->getDriver();
    $assert_session = $this->assertSession();
    $this->drupalGet('/admin/structure/types/manage/page/form-display');
    $driver->selectOption('//select[@name = "fields[field_example][type]"]', $widget);
    $assert_session->assertWaitOnAjaxRequest();
    $driver->click('//input[@value = "Save"]');
    $assert_session->statusMessageContains('Your settings have been saved.');
  }

  /**
   * {@selfdoc}
   */
  final protected function configureCustomFormatter(): void {
    $driver = $this->getSession()->getDriver();
    $assert_session = $this->assertSession();
    $this->drupalGet('/admin/structure/types/manage/page/display');
    $driver->selectOption('//select[@name = "fields[field_example][type]"]', 'date_point_custom');
    $assert_session->assertWaitOnAjaxRequest();
    $driver->click('//tr[@id = "field-example"]//input[@name = "field_example_settings_edit"]');
    $assert_session->assertWaitOnAjaxRequest();
    $driver->setValue('//label[text() = "Date/time format"]/following-sibling::input', 'Y-m-d H:i:s.u');
    $driver->click('//input[@name = "field_example_plugin_settings_update"]');
    $assert_session->assertWaitOnAjaxRequest();
    $driver->click('//input[@value = "Save"]');
    $assert_session->statusMessageContains('Your settings have been saved.');
  }

  /**
   * {@selfdoc}
   */
  final protected function createPage(?string $datetime): string {
    $driver = $this->getSession()->getDriver();
    $assert_session = $this->assertSession();
    $this->drupalGet('/node/add/page');
    $assert_session->fieldValueEquals('field_example[0][value]', '');
    $driver->setValue('//input[@name = "title[0][value]"][1]', 'Test');
    if ($datetime) {
      $driver->evaluateScript(
        \sprintf('document.querySelector(\'input[name="field_example[0][value]"]\').value = "%s";', $datetime),
      );
    }
    $driver->click('//input[@value = "Save"]');
    $assert_session->statusMessageContains('Page Test has been created.');
    $url_parts = \explode('/', $this->getSession()->getCurrentUrl());
    return \end($url_parts);
  }

  /**
   * {@selfdoc}
   */
  final protected function assertDisplayedDatetime(string $expected_output): void {
    $driver = $this->getSession()->getDriver();
    try {
      $actual_output = $driver->getText('//div[text() = "Example"]/following-sibling::div/time');
    }
    // @phpstan-ignore-next-line
    catch (NoSuchElement) {
      self::fail('Could not find formatted date.');
    }
    self::assertSame($expected_output, $actual_output);
  }

  /**
   * {@selfdoc}
   */
  final protected function deleteAllNodes(): void {
    $node_storage = $this->container->get('entity_type.manager')->getStorage('node');
    $driver = $this->getSession()->getDriver();
    foreach ($node_storage->loadMultiple() as $node) {
      $this->drupalGet(\sprintf('/node/%d/delete', $node->id()));
      $driver->click('//input[@value = "Delete"]');
    }
  }

}
